@@ -51,7 +51,8 @@ struct c_connection
 	int follow_nick;
 	int ignore_first_nick;
 	char *away_nick;
-	char *on_connect_send;
+	list_t on_connect_send;
+	char *no_client_away_msg;
 
 	struct client *client;
 };
@@ -65,7 +66,7 @@ struct c_channel
 #ifdef HAVE_LIBSSL
 int adm_trust(struct link_client *ic, struct line *line);
 #endif
-int adm_bip(struct link_client *ic, struct line *line);
+int adm_bip(struct link_client *ic, struct line *line, unsigned int privmsg);
 int ssl_check_trust(struct link_client *ic);
 void adm_blreset(struct link_client *ic);
 
