@@ -28,6 +28,8 @@ int conf_memlog = 1;
 extern int conf_backlog;
 extern int conf_backlog_lines;
 extern int conf_always_backlog;
+extern int conf_bl_msg_only;
+extern int conf_backlog_no_timestamp;
 
 int log_set_backlog_offset(log_t *logdata, char *dest);
 static int _log_write(log_t *logdata, logfilegroup_t *lf, char *d, char *str);
@@ -66,7 +68,7 @@ int check_dir_r(char *dirname)
 	int pos, count = 0;
 	char *dir, *tmp;
 	int len = strlen(dirname);
-	
+
 	mylog(LOG_DEBUGVERB, "Recursive check of %s engaged", dirname);
 	tmp = dirname;
 	dir = (char *)malloc(sizeof(char) * (len + 1));
@@ -131,12 +133,12 @@ char *log_build_filename(log_t *logdata, char *destination)
 	time_t s;
 	char *dest = strdup(destination);
 	strtolower(dest);
-	
+
 	log_format_len = strlen(conf_log_format);
 	logfile = (char*)malloc((MAX_PATH_LEN + 1)*sizeof(char));
 	if (!logfile)
 		fatal("out of memory");
-	
+
 	time(&s);
 	now = localtime(&s);
 	snprintf(year, 5, "%04d", now->tm_year + 1900);
@@ -150,16 +152,16 @@ char *log_build_filename(log_t *logdata, char *destination)
 	replace_var(logfile, "%Y", year, MAX_PATH_LEN);
 	replace_var(logfile, "%d", day, MAX_PATH_LEN);
 	replace_var(logfile, "%m", month, MAX_PATH_LEN);
-	
+
 	logdir = strdup(logfile);
 	if (!logdir)
 		fatal("out of memory");
-	
+
 	/* strrchr works on bytes, not on char (if sizeof(char) != 1) */
 	tmp = strrchr(logdir, '/');
 	if (tmp)
 		*tmp = '\0';
-	
+
 	free(dest);
 	if (check_dir_r(logdir)) {
 		free(logfile);
@@ -611,7 +613,7 @@ void log_client_none_connected(log_t *logdata)
 	if (conf_always_backlog)
 		return;
 
-	
+
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
 			hash_it_next(&hi)) {
 		lfg = hash_it_item(&hi);
@@ -699,7 +701,6 @@ int log_has_backlog(log_t *logdata, char *destination)
  * 01-08-2005 10:46:11 < * jj!john@thebox.ofjj.net
  */
 
-/* must *not* return NULL */
 char *log_beautify(char *buf, char *dest)
 {
 	int action = 0;
@@ -715,7 +716,7 @@ char *log_beautify(char *buf, char *dest)
 	int done;
 
 	if (!buf)
-		mylog(LOG_DEBUG, "BUG!");
+		fatal("BUG log_beautify not called correctly!");
 
 	p = strchr(buf, ' ');
 	if (!p || !p[0] || !p[1])
@@ -727,8 +728,12 @@ char *log_beautify(char *buf, char *dest)
 		return _log_wrap(dest, buf);
 	lots = p - sots;
 	p++;
-	if (strncmp(p, "-!-", 3) == 0)
-		return _log_wrap(dest, buf);
+	if (strncmp(p, "-!-", 3) == 0) {
+		if (conf_bl_msg_only)
+			return NULL;
+		else
+			return _log_wrap(dest, buf);
+	}
 
 	if (*p == '>')
 		out = 1;
@@ -736,6 +741,7 @@ char *log_beautify(char *buf, char *dest)
 		out = 0;
 	else
 		return _log_wrap(dest, buf);
+
 	p++;
 	if (*p != ' ')
 		return _log_wrap(dest, buf);
@@ -746,7 +752,7 @@ char *log_beautify(char *buf, char *dest)
 			return _log_wrap(dest, buf);
 		p += 2;
 	}
-	
+
 	son = p;
 	while (*p && *p != '!' && *p != ' ' && *p != ':')
 		p++;
@@ -757,10 +763,10 @@ char *log_beautify(char *buf, char *dest)
 	p = strchr(p, ' ');
 	if (!p || !p[0])
 		return _log_wrap(dest, buf);
-	
+
 	done = ((p[-1] == ':') || (action && (p[1] != '(')));
 	p++;
-	
+
 	/*
 	 * TODO add a : before the action text in the log files
 	 * otherwise "/me (bla) blabla" on a chan is logged exactly as
@@ -774,7 +780,7 @@ char *log_beautify(char *buf, char *dest)
 		while (*p && *p != ')' && *p != ' ' && *p != '!')
 			p++;
 		lod = p - sod;
-		
+
 		if (*p == '!')
 			while (*p && *p != ')' && *p != ' ')
 				p++;
@@ -808,7 +814,7 @@ char *log_beautify(char *buf, char *dest)
 		return _log_wrap(dest, buf);
 
 	p = ret = (char *)malloc(
-		1 + lon + strlen(LAMESTRING) + lod + 2 + lots +	1 + lom + 3
+		1 + lon + strlen(LAMESTRING) + lod + 2 + lots +	2 + lom + 3
 		+ action * (2 + strlen("ACTION ")) + out * strlen(" -> "));
 	if (!p)
 		fatal("out of memory");
@@ -836,24 +842,26 @@ char *log_beautify(char *buf, char *dest)
 		strcpy(p, " -> ");
 		p += strlen(" -> ");
 	}
-	memcpy(p, sots, lots);
-	p += lots;
-
-	*p++ = ' ';
+	if (conf_backlog_no_timestamp == 0) {
+		memcpy(p, sots, lots);
+		p += lots;
+		*p++ = '>';
+		*p++ = ' ';
+	}
 
 	memcpy(p, som, lom);
 	p += lom;
 
 	if (action)
 		*p++ = 1;
-	
+
 	*p++ = '\r';
 	*p++ = '\n';
 	*p = 0;
 	return ret;
 }
 
-char *log_backread(log_t *logdata, char *destination)
+char *log_backread(log_t *logdata, char *destination, int *skip)
 {
 	char *buf;
 	size_t pos = 0;
@@ -862,6 +870,8 @@ char *log_backread(log_t *logdata, char *destination)
 	int c;
 	char *ret;
 
+	*skip = 0;
+
 	if (!conf_always_backlog && logdata->connected)
 		return NULL;
 
@@ -898,12 +908,12 @@ next_file:
 	/* check the files containing data to backlog */
 	lf = list_it_item(&logdata->file_it);
 	if (lf != list_get_last(&lfg->file_group)) {
-		mylog(LOG_DEBUG, "%s not last file!", lf->filename);
+		mylog(LOG_DEBUGVERB, "%s not last file!", lf->filename);
 		/* if the file is not the current open for logging
 		 * (it is an old file that has been rotated)
 		 * open if necessary, backlog line per line, and close */
 		if (!lf->file) {
-			mylog(LOG_DEBUG, "opening: %s!", lf->filename);
+			mylog(LOG_DEBUGVERB, "opening: %s!", lf->filename);
 			lf->file = fopen(lf->filename, "r");
 			if (!lf->file) {
 				mylog(LOG_ERROR, "Can't open %s for reading",
@@ -913,12 +923,13 @@ next_file:
 				return _log_wrap("Error reading logfile",
 						destination);
 			}
-			mylog(LOG_DEBUG, "seeking: %d!", lf->backlog_offset);
+			mylog(LOG_DEBUGVERB, "seeking: %d!",
+					lf->backlog_offset);
 			if (fseek(lf->file, lf->backlog_offset, SEEK_SET)) {
 				log_reinit(lfg);
 				free(buf);
-				return _log_wrap("Error reading in logfile",
-						destination);
+				return _log_wrap(destination,
+						"Error reading in logfile");
 			}
 		}
 		for(;;) {
@@ -932,7 +943,7 @@ next_file:
 				if (pos == LOGLINE_MAXLEN)
 					mylog(LOG_DEBUG, "logline too long");
 				if (c == EOF || pos == LOGLINE_MAXLEN) {
-					mylog(LOG_DEBUG, "EOF: %s (%d)!",
+					mylog(LOG_DEBUGVERB, "EOF: %s (%d)!",
 							lf->filename,
 							conf_always_backlog);
 
@@ -950,7 +961,15 @@ next_file:
 					goto next_file;
 				}
 				buf[pos] = 0;
+				if (pos == 0) {
+					*skip = 1;
+					return buf;
+				}
 				ret = log_beautify(buf, destination);
+				if (ret == NULL) {
+					pos = 0;
+					continue;
+				}
 				free(buf);
 				return ret;
 			}
@@ -962,8 +981,8 @@ next_file:
 	if (!logdata->lastfile_seeked) {
 		if (fseek(lf->file, lf->backlog_offset, SEEK_SET)) {
 			log_reinit(lfg);
-			return _log_wrap("Error reading in logfile",
-					destination);
+			return _log_wrap(destination,
+					"Error reading in logfile");
 		}
 		logdata->lastfile_seeked = 1;
 		mylog(LOG_DEBUG, "last file seedked!");
@@ -996,10 +1015,18 @@ next_file:
 				return NULL;
 			}
 
-			if (conf_always_backlog && c == EOF)
+			if (!conf_always_backlog && c == EOF)
 				lf->backlog_offset--;
 			buf[pos] = 0;
+			if (pos == 0) {
+				*skip = 1;
+				return buf;
+			}
 			ret = log_beautify(buf, destination);
+			if (ret == NULL) {
+				pos = 0;
+				continue;
+			}
 			free(buf);
 			return ret;
 		}
@@ -1028,7 +1055,7 @@ static char *_log_wrap(char *dest, char *line)
 }
 
 static int _log_write(log_t *logdata, logfilegroup_t *lfg, char *destination,
-		char *str) 
+		char *str)
 {
 	size_t nbwrite;
 	size_t len;
@@ -1037,11 +1064,13 @@ static int _log_write(log_t *logdata, logfilegroup_t *lfg, char *destination,
 
 	if (lfg->memlog) {
 		char *r = log_beautify(str, destination);
-		list_add_last(lfg->memlog, r);
-		if (lfg->memc == conf_backlog_lines)
-			free(list_remove_first(lfg->memlog));
-		else
-			lfg->memc++;
+		if (r != NULL) {
+			list_add_last(lfg->memlog, r);
+			if (lfg->memc == conf_backlog_lines)
+				free(list_remove_first(lfg->memlog));
+			else
+				lfg->memc++;
+		}
 	}
 
 	if (!conf_log)
