@@ -384,7 +384,7 @@ void list_free(list_t *t)
 void list_append(list_t *src, list_t *dest)
 {
 	list_iterator_t it;
-	
+
 	for (list_it_init(src, &it); list_it_item(&it); list_it_next(&it))
 		list_add_last(dest, list_it_item(&it));
 }
@@ -396,7 +396,7 @@ void list_append(list_t *src, list_t *dest)
 struct hash_item {
 	char *key;
 	void *item;
-}; 
+};
 
 static int hash_item_nocase_cmp(struct hash_item *a, char *b)
 {
@@ -453,12 +453,15 @@ hash_t *hash_new(int options)
 	return h;
 }
 
+/* Now we have a real hash, but we use only the last byte of it :p */
 static unsigned char hash_func(char *pkey)
 {
-	unsigned char i = 0;
-	while (*pkey)
-		i += (unsigned char)toupper(*pkey++);
-	return i;
+	char c;
+	unsigned long hash = 5381; /* 5381 & 0xff makes more sense */
+
+	while ((c = *pkey++))
+		hash = ((hash << 5) + hash) ^ toupper(c);
+	return (unsigned char)hash;
 }
 
 void hash_insert(hash_t *hash, char *key, void *ptr)
@@ -467,7 +470,7 @@ void hash_insert(hash_t *hash, char *key, void *ptr)
 
 	if (hash_get(hash, key))
 		fatal("Element with key %s already in hash %x\n", key, hash);
-	
+
 	it = malloc(sizeof(struct hash_item));
 	if (!it)
 		fatal("malloc");
@@ -550,12 +553,18 @@ char *hash_it_key(hash_iterator_t *h)
 	return hi->key;
 }
 
+void hash_dump(hash_t *h)
+{
+	hash_iterator_t it;
+	for (hash_it_init(h, &it); hash_it_item(&it) ;hash_it_next(&it))
+		printf("%s => %p\n", hash_it_key(&it), hash_it_item(&it));
+}
+
 char *strmaydup(char *s)
 {
 	if (!s)
 		return s;
 	return strdup(s);
-	
 }
 
 void strucase(char *s)
